<!--
layout: home
title: Cosmos SDK Tutorials
description: Tutorials for the Cosmos SDK
sections:
  - title: Create a Blog module
    desc: Learn how to create a blog module using the Cosmos SDK blockchain app. Scaffold a new blockchain. Create a blog module and build your own transaction types.
    url: /blog/tutorial/01-index.html
    tags: 
      - starport
      - stargate
  - title: Create a Voting module
    desc: Learn to create a voter module using the Cosmos SDK blockchain app. Create Polls and have accounts Vote on your blockchain.
    url: /voter/index.html
    tags: 
      - starport
      - stargate
  - title: Create a IBC Hello World module
    desc: Build a Hello World blockchain app as an IBC module. Learn how to use the relayer to connect two blockchains with each other and use IBC packets to transfer data from one blockchain to another.
    url: /hello-world/tutorial/
    tags: 
      - starport
      - stargate
  - title: Create an IBC Interchain Exchange module
    desc: Build an Exchange blockchain app as an IBC module. You will build a module that can create orderbooks and buy and sell orders across blockchains. Learn how to use IBC, IBC packets, and the relayer.
    url: /interchain-exchange/tutorial/00-intro.html
    tags: 
      - starport
      - stargate
  - title: Connect to the cosmoshub-testnet
    desc: Learn how to connect to the cosmoshub-testnet and use the official cosmos sdk modules, from governance, bank, auth, ibc to the newest liquidity module.
    url: /connecting-to-testnet/index.html
    tags: 
      - starport
      - stargate
  - title: Understand IBC denoms
    desc: In this tutorial, you learn what IBC denoms are, how to use the relayer, connect blockchains, and find the path to an unknown blockchain.
    url: /understanding-ibc-denoms/index.html
    tags: 
      - gaia
      - stargate
  - title: Understand the liquidity module
    desc: The liquidity module powers the Gravity DEX. Learn how to create your own token with Starport, send it with IBC to the cosmoshub-testnet and create a new pool using the liquidity module. Swap your token with your created pool.
    url: /liquidity-module/index.html
    tags: 
      - starport
      - stargate
  - title: Deploy your blockchain on Digital Ocean
    desc: Create a new blockchain with Starport. Deploy this blockchain on a cloud provider like Digital Ocean. Connect your local chain to the deployed chain.
    url: /publish-app-do/index.html
    tags: 
      - starport
      - stargate
  - title: Migrate a blockchain - Proof of File Existence example
    desc: Efficiently migrate your Cosmos SDK applications or modules from v0.39 to v0.40+ using Starport.
    url: /launchpad-to-stargate/tutorial/01-introduction.html
    tags:
      - starport
      - stargate
stack:
  - title: "Code with Us: Dive into Starport, IBC and Gravity DEX Protocol"
    duration: "1:00:27"
    imgSrc: https://i.ytimg.com/vi/GxaqpzMk0jk/hq720.jpg
    url: https://youtu.be/GxaqpzMk0jk
  - title: "Starport v0.16: Sending tokens to Cosmos Hub with IBC"
    duration: "6:28"
    imgSrc: https://i.ytimg.com/vi/-AuExRijtrA/hq720.jpg
    url: https://youtu.be/-AuExRijtrA
  - title: "Starport Intro"
    duration: "4:12"
    imgSrc: https://i.imgur.com/JmyOl8n.jpeg
    url: https://youtu.be/5RqAIE0b8Kw
  - title: "CosmWasm & IBC - The New Horizon"
    duration: "44:32"
    imgSrc: https://i.ytimg.com/vi/42eePmGk6D8/hq720.jpg
    url: https://youtu.be/42eePmGk6D8
  - title: "Blockchain Performance Measured Correctly"
    duration: "1:16:56"
    imgSrc: https://i.ytimg.com/vi/QdKTkJHj4jY/hq720.jpg
    url: https://www.youtube.com/watch?v=QdKTkJHj4jY
  - title: "Starport v0.15: IBC Hello, world!"
    duration: "10:21"
    imgSrc: https://i.ytimg.com/vi/NmytpuD33lY/hq720.jpg
    url: https://www.youtube.com/watch?v=NmytpuD33lY
  - title: Cosmos Code With Us - Building your first Cosmos app
    duration: "1:39:07"
    imgSrc: https://i.ytimg.com/vi/h6Ur_40LB9k/hq720.jpg
    url: https://www.youtube.com/watch?v=h6Ur_40LB9k
  - title: Getting started with Starport, the easiest way to build a Cosmos SDK blockchain
    duration: "3:31"
    imgSrc: https://i.ytimg.com/vi/rmbPjCGDXek/hq720.jpg
    url: https://www.youtube.com/watch?v=rmbPjCGDXek
  - title: Building a PoFE blockchain with Starport
    duration: "56:28"
    imgSrc: https://i.ytimg.com/vi/PGLAW-HrzWg/hq720.jpg
    url: https://www.youtube.com/watch?v=PGLAW-HrzWg
  - title: Code with us - Scavenger Hunt Tutorial with Billy Rennekamp
    duration: "2:11:15"
    imgSrc: https://i.ytimg.com/vi/GfZjnMchKX0/hq720.jpg
    url: https://www.youtube.com/watch?v=GfZjnMchKX0
  - title: Cosmos Code With Us - Front end name service application
    duration: "1:59:34"
    imgSrc: https://i.ytimg.com/vi/ooc9ODGxqcA/hq720.jpg
    url: https://www.youtube.com/watch?v=ooc9ODGxqcA
footer:
  newsletter: false
aside: false
-->

# Cosmos SDK Tutorials

The [Cosmos SDK Tutorials](https://tutorials.cosmos.network/) teach you how to use Cosmos SDK, the world’s most popular framework for building application-specific blockchains. Tutorials provide step-by-step instructions to help you build foundational knowledge and learn how to use Starport and the Cosmos SDK, including: 

- Foundational knowledge to help you navigate between blockchains with the Cosmos SDK
- Learn how Starport works
- Create a blockchain polling application
- Build an exchange that works with two or more blockchains
- Interact with the Cosmos Hub testnet to test the functionality of your blockchain
- Use the liquidity module, known on the Cosmos Hub as Gravity DEX, to create liquidity pools so users can swap tokens
- Publish your blockchain application to a Droplet on DigitalOcean

The code and docs for each tutorial are based on a specific version of the software. Be sure to follow the tutorial instructions to download and use the right version.

Use the tutorials landing page as your entry point to articles on [Cosmos blog](https://blog.cosmos.network/), videos on [Cosmos YouTube](https://www.youtube.com/c/CosmosProject/videos), and ways to get help and support. 


This repo manages and publishes the tutorials. For details, see [CONTRIBUTING](CONTRIBUTING.md). 
