---
order: 15
---

# NameService Module Rest Interface

Your module can also expose a REST interface to allow programatic access to the module's functionality. To get started navigate to `./x/nameservice/client/rest/rest.go` where HTTP handlers are held.

> _*NOTE*_: Your application needs to import the code you just wrote. Here the import path is set to this repository (`github.com/cosmos/sdk-tutorials/nameservice/x/nameservice`). If you are following along in your own repo you will need to change the import path to reflect that (`github.com/{ .Username }/{ .Project.Repo }/x/nameservice`).

### RegisterRoutes

First, define the REST client interface for your module in a `RegisterRoutes` function. Have the routes all starting with your module name to prevent name space collisions with other modules' routes:

<<< @/nameservice/nameservice/x/nameservice/client/rest/rest.go

### Query Handlers

Next, its time to define the handlers mentioned above in `queryWhois.go`. These will be very similar to the CLI methods defined earlier. `listWhoisHandler` and `getWhoisHandler` should already be defined, and you can use `getWhois` as a template to write the `resolveNameHandler` function.

<<< @/nameservice/nameservice/x/nameservice/client/rest/queryWhois.go

Notes on the above code:

- Notice we are using the same `cliCtx.QueryWithData` function to fetch the data
- These functions are almost the same as the corresponding CLI functionality

### Tx Handlers

Now define the `buyName`, `setName` and `deleteName` transaction routes in `txWhois.go` - you can replace the existing handlers that were generated by `starport type`. Notice these aren't actually performing the transactions to buy, set and delete names, as in a general case this would require some form of authentication. Instead these endpoints build and return each specific transaction which can then be signed in a secure manner and afterwards broadcast to the network using a standard endpoint like `/txs`.

<<< @/nameservice/nameservice/x/nameservice/client/rest/txWhois.go

Notes on the above code:

- The [`BaseReq`](https://godoc.org/github.com/cosmos/cosmos-sdk/client/utils#BaseReq) contains the basic required fields for making a transaction (which key to use, how to decode it, which chain you are on, etc...) and is designed to be embedded as shown.
- `baseReq.ValidateBasic` handles setting the response code for you and therefore you don't need to worry about handling errors or successes when using those functions.

### Next its time to augment `nameservice` by implementing the AppModule interface.
